import { highlightAll } from 'prismjs';
import { isPlatformBrowser, CommonModule } from '@angular/common';
import { InjectionToken, Inject, PLATFORM_ID, ElementRef, Input, Component, ViewEncapsulation, NgModule, Optional, Injectable } from '@angular/core';
import { HttpResponse, HttpClient, HttpClientModule } from '@angular/common/http';
import marked from 'marked';
import { of, Observable } from 'rxjs';
import { map, catchError } from 'rxjs/operators';

var NT_MARKDOWN_ENGINE = new InjectionToken('nt-markdown-engine');

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var NtMarkdownComponent = /** @class */ (function () {
    function NtMarkdownComponent(_platformId, _markdownEngine, _elementRef) {
        this._platformId = _platformId;
        this._markdownEngine = _markdownEngine;
        this._elementRef = _elementRef;
    }
    Object.defineProperty(NtMarkdownComponent.prototype, "path", {
        get: function () { return this._path; },
        set: function (value) { this._path = value || ''; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtMarkdownComponent.prototype, "data", {
        get: function () { return this._data; },
        set: function (value) { this._data = value; },
        enumerable: true,
        configurable: true
    });
    NtMarkdownComponent.prototype.ngOnChanges = function (changes) {
        var pathChange = changes.path;
        var dataChange = changes.data;
        if (pathChange && !pathChange.firstChange) {
            this._pathChange();
        }
        if (dataChange && !dataChange.firstChange) {
            this._dataChange();
        }
    };
    /*
     *  After view init
     */
    NtMarkdownComponent.prototype.ngAfterViewInit = function () {
        if (this.path) {
            this._pathChange();
        }
        else {
            this._dataChange();
        }
    };
    // on input
    NtMarkdownComponent.prototype._dataChange = function () {
        var _this = this;
        if (this.data) {
            this._markdownEngine.compile(this.data).subscribe(function (markdown) { return _this._setMarkdownHtml(markdown); });
        }
        else {
            this._setMarkdownHtml('');
        }
    };
    /**
     * get remote conent;
     */
    NtMarkdownComponent.prototype._pathChange = function () {
        var _this = this;
        if (this.path) {
            this._ext = this.path && this.path.split('.').splice(-1).join();
            this._markdownEngine.getContent(this.path).subscribe(function (content) {
                content = _this._ext !== 'md' ? ['```', _this._ext + "\n" + content + "\n", '```'].join() : content;
                _this._markdownEngine.compile(_this._prepare(content)).subscribe(function (markdown) { return _this._setMarkdownHtml(markdown); });
            }, function (error) { return _this._handleError(error); });
        }
    };
    /**
     * catch http error
     */
    NtMarkdownComponent.prototype._handleError = function (error) {
        return Promise.reject(error.message || error);
    };
    /**
     * Prepare string
     */
    NtMarkdownComponent.prototype._prepare = function (raw) {
        var _this = this;
        if (!raw) {
            return '';
        }
        if (this._ext === 'md' || !this.path) {
            var isCodeBlock_1 = false;
            return raw.split('\n').map(function (line) {
                if (_this._trimLeft(line).substring(0, 3) === "```") {
                    isCodeBlock_1 = !isCodeBlock_1;
                }
                return isCodeBlock_1 ? line : line.trim();
            }).join('\n');
        }
        return raw.replace(/\"/g, '\'');
    };
    NtMarkdownComponent.prototype._postpare = function () {
        // 判断是否是浏览器执行，如果是：增加代码高亮
        if (isPlatformBrowser(this._platformId)) {
            highlightAll(false);
        }
    };
    NtMarkdownComponent.prototype._setMarkdownHtml = function (markdown) {
        this._elementRef.nativeElement.innerHTML = markdown;
        this._postpare();
    };
    /**
     * Trim left whitespace
     */
    NtMarkdownComponent.prototype._trimLeft = function (line) {
        return line.replace(/^\s+|\s+$/g, '');
    };
    NtMarkdownComponent.ctorParameters = function () { return [
        { type: Object, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] },
        { type: undefined, decorators: [{ type: Inject, args: [NT_MARKDOWN_ENGINE,] }] },
        { type: ElementRef }
    ]; };
    __decorate([
        Input()
    ], NtMarkdownComponent.prototype, "path", null);
    __decorate([
        Input()
    ], NtMarkdownComponent.prototype, "data", null);
    NtMarkdownComponent = __decorate([
        Component({
            selector: 'nt-markdown, [nt-markdown]',
            template: '',
            encapsulation: ViewEncapsulation.None,
            host: {
                class: 'nt-markdown'
            }
        }),
        __param(0, Inject(PLATFORM_ID)),
        __param(1, Inject(NT_MARKDOWN_ENGINE))
    ], NtMarkdownComponent);
    return NtMarkdownComponent;
}());

var __decorate$1 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var NtMarkdownModule = /** @class */ (function () {
    function NtMarkdownModule() {
    }
    NtMarkdownModule = __decorate$1([
        NgModule({
            imports: [CommonModule],
            declarations: [NtMarkdownComponent],
            exports: [NtMarkdownComponent]
        })
    ], NtMarkdownModule);
    return NtMarkdownModule;
}());

var NT_MARKED_DEFAULT_OPTIONS = {
    gfm: true,
    tables: true,
    breaks: false,
    pedantic: false,
    sanitize: true,
    smartLists: true,
    smartypants: false
};
var NT_MARKED_OPTIONS = new InjectionToken('nt-marked-options');

var __decorate$2 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param$1 = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var NtMarkedEngine = /** @class */ (function () {
    function NtMarkedEngine(http, options) {
        if (options === void 0) { options = {}; }
        this.http = http;
        this._renderer = new marked.Renderer();
        this.extendRenderer();
        this.setMarkedOptions(options);
    }
    Object.defineProperty(NtMarkedEngine.prototype, "renderer", {
        get: function () { return this._renderer; },
        enumerable: true,
        configurable: true
    });
    // get the content from remote resource
    NtMarkedEngine.prototype.getContent = function (path) {
        return this.http.get(path).pipe(map(this.extractData), catchError(this.handleError));
    };
    // handle data
    NtMarkedEngine.prototype.extractData = function (res) {
        return res.body || '';
    };
    NtMarkedEngine.prototype.setMarkedOptions = function (options) {
        options.renderer = this._renderer;
        marked.setOptions(options);
    };
    // comple markdown to html
    NtMarkedEngine.prototype.compile = function (data) { return of(marked(data)); };
    // handle error
    NtMarkedEngine.prototype.handleError = function (error) {
        var errMsg;
        if (error instanceof HttpResponse) {
            var body = error.body || '';
            var err = body.error || JSON.stringify(body);
            errMsg = error.status + " - " + (error.statusText || '') + " " + err;
        }
        else {
            errMsg = error.message ? error.message : error.toString();
        }
        return Observable.throw(errMsg);
    };
    // extend marked render to support todo checkbox
    NtMarkedEngine.prototype.extendRenderer = function () {
        // this._renderer.image = function (href: string, title: string, text: string) {
        // 	return `<img href="${href}" alt="${title}">`;
        // };
        this._renderer.heading = function (text, level) {
            text = text.trim();
            return "<h" + level + " id=\"" + text.replace(/\<a[^>]*?\>((.|\n)*?)\<\/a\>/g, '$1').replace(/ +/g, '-') + "\">\n\t\t\t\t\t\t" + text + "\n\t\t\t\t\t</h" + level + ">";
        };
        this._renderer.listitem = function (text) {
            if (/^\s*\[[x ]\]\s*/.test(text)) {
                var checkedStyle = 'vertical-align: middle; margin: 0 0.2em 0.25em -1.6em; font-size: 16px;';
                text = text
                    .replace(/^\s*\[ \]\s*/, "<input type=\"checkbox\" style=\"" + checkedStyle + "\" disabled>")
                    .replace(/^\s*\[x\]\s*/, "<input type=\"checkbox\" style=\"" + checkedStyle + "\" checked disabled>");
                return "<li style=\"list-style: none\">" + text + "</li>";
            }
            else {
                return "<li>" + text + "</li>";
            }
        };
    };
    NtMarkedEngine.ctorParameters = function () { return [
        { type: HttpClient },
        { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [NT_MARKED_OPTIONS,] }] }
    ]; };
    NtMarkedEngine = __decorate$2([
        Injectable(),
        __param$1(1, Optional()), __param$1(1, Inject(NT_MARKED_OPTIONS))
    ], NtMarkedEngine);
    return NtMarkedEngine;
}());

var __decorate$3 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var ɵ0 = NT_MARKED_DEFAULT_OPTIONS;
var NtMarkedEngineModule = /** @class */ (function () {
    function NtMarkedEngineModule() {
    }
    NtMarkedEngineModule = __decorate$3([
        NgModule({
            imports: [HttpClientModule],
            providers: [
                { provide: NT_MARKDOWN_ENGINE, useClass: NtMarkedEngine },
                { provide: NT_MARKED_OPTIONS, useValue: ɵ0 },
            ],
        })
    ], NtMarkedEngineModule);
    return NtMarkedEngineModule;
}());

/*
 * Public API Surface of markdown
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NT_MARKDOWN_ENGINE, NT_MARKED_DEFAULT_OPTIONS, NT_MARKED_OPTIONS, NtMarkdownComponent, NtMarkdownModule, NtMarkedEngine, NtMarkedEngineModule, ɵ0 };
//# sourceMappingURL=markdown.js.map
